Package: venn
Version: 1.13
Title: Draw Venn Diagrams
Authors@R: person(given = "Adrian", family = "Dusa",
    role = c("aut", "cre", "cph"),
    email = "dusa.adrian@unibuc.ro",
    comment = c(ORCID = "0000-0002-3525-9253"))
Depends: R (>= 3.5.0)
Imports: admisc (>= 0.33)
Suggests: QCA (>= 3.9), ggplot2, ggpolypath
Description: A close to zero dependency package to draw and display Venn
    diagrams up to 7 sets, and any Boolean union of set intersections.
License: GPL (>= 3)
URL: https://github.com/dusadrian/venn
BugReports: https://github.com/dusadrian/venn/issues
NeedsCompilation: no
Packaged: 2026-03-31 06:28:41 UTC; dusadrian
Author: Adrian Dusa [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-3525-9253>)
Maintainer: Adrian Dusa <dusa.adrian@unibuc.ro>
Repository: CRAN
Date/Publication: 2026-03-31 06:50:02 UTC
